﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using System.Text;
using System.Threading.Tasks;

namespace LinqExpressionFilterIssue
{
    class Program
    {
        static void Main(string[] args)
        {
            // sample data
            List<Event> Events = new List<Event>()
            {
                new Event() { Id = 1, Field1 = "Cat", Field2 = 1},
                new Event() { Id = 1, Field1 = "Cat", Field2 = 2},
                new Event() { Id = 1, Field1 = "Mouse", Field2 = 2},
                new Event() { Id = 1, Field1 = "Cat", Field2 = 3},
                new Event() { Id = 1, Field1 = "Mouse", Field2 = 3},
                new Event() { Id = 1, Field1 = "Cat", Field2 = 4}
            };

            // sample VALS
            // i intentionaly left out 2
            List<int> vals = new List<int>()
            {
                1,  3, 4
            };

            // the equivilent of your Search method
            // asuming filter = "cat"

            Expression<Func<Event, bool>> filter = x => x.Field1 == "Cat";

            /* At this point output will be
                ID= 1, Field1= Cat, Field2= 1
                ID= 1, Field1= Cat, Field2= 2
                ID= 1, Field1= Cat, Field2= 3
                ID= 1, Field1= Cat, Field2= 4
             * 
               * Query: SELECT * FROM SOMTABLE WHERE Field1 = 'Cat'
             */

            
            if (vals.Count > 0)
            {
                Expression<Func<Event, bool>> AndFilter = x => x.Field2 == vals.First();
                filter = filter.And(AndFilter);


                /* At this point output will be
                    ID= 1, Field1= Cat, Field2= 1
                 * 
                 * Query: SELECT * FROM SOMTABLE WHERE Field1 = 'Cat' AND Field2 = 1
                 */
            }
            
            
            if (vals.Count > 1)
            {
                Expression<Func<Event, bool>> OrFilter = x => vals.Skip(1).Contains(x.Field2);
                filter = filter.Or(OrFilter);

                // NOTE: "2" is not in out vals array
                /* At this point output will be
                    ID= 1, Field1= Cat, Field2= 1
                    ID= 1, Field1= Cat, Field2= 3
                    ID= 1, Field1= Mouse, Field2= 3
                    ID= 1, Field1= Cat, Field2= 4
                 * 
                 * Query: SELECT * FROM SOMTABLE WHERE Field1 = 'Cat' AND (Field2 = 1 OR Field2 = 3 OR Field2 = 4)
                 */
            }
            


            foreach(var item in Events.AsQueryable().Where(filter).ToList())
            {
                Console.WriteLine(string.Concat("ID= ", item.Id.ToString(), ", Field1= ", item.Field1, ", Field2= ", item.Field2.ToString()));

            }

            Console.ReadKey();
        }


    }
    public class Event
    {
        public int Id { get; set; }
        public string Field1 { get; set; }
        public int Field2 { get; set;  }
    }
}
